@@ -2,8 +2,18 @@ package XML::Validator::Schema::Attribute;
 use strict;
 use warnings;
 
-use XML::Validator::Schema::Util qw(_attr);
-use XML::Validator::Schema::Type qw(supported_type);
+=head1 NAME
+
+XML::Validator::Schema::Attribute - an attribute node in a schema object
+
+=head1 DESCRIPTION
+
+This is an internal module used by XML::Validator::Schema to represent
+attributes derived from XML Schema documents.
+
+=cut
+
+use XML::Validator::Schema::Util qw(_attr _err);
 
 sub new {
     my ($pkg, %arg) = @_;
@@ -16,31 +26,24 @@ sub parse {
     my $self = $pkg->new();
 
     my $name = _attr($data, 'name');
-    $self->_err('Found <attribute> without a name.')
+    _err('Found <attribute> without a name.')
       unless $name;
     $self->{name} = $name;
 
-    my $type = _attr($data, 'type');
-    if ($type) {
-        $self->_err(qq{Attribute '$name' has an unsupported type: $type.})
-          unless supported_type($type);
-        $self->{type} = $type;
+    my $type_name = _attr($data, 'type');
+    if ($type_name) {
+        $self->{unresolved_type} = 1;
+        $self->{type_name} = $type_name;
     }
 
     # load use, defaults to optional
     my $use = _attr($data, 'use') || 'optional';
-    $self->_err("Invalid 'use' value in <attribute name='$name'>: '$use'.") 
+    _err("Invalid 'use' value in <attribute name='$name'>: '$use'.") 
       unless $use eq 'optional' or $use eq 'required';
     $self->{required} = $use eq 'required' ? 1 : 0;
 
     return $self;
 }
 
-# throw a Validator exception
-sub _err {
-    my $self = shift;
-    XML::SAX::Exception::Validator->throw(Message => shift);
-}
-
 1;
 
