@@ -16,7 +16,7 @@ Schema documents.
 
 use base 'XML::SAX::Base';
 use Data::Dumper;
-use XML::Validator::Schema::Util qw(_attr);
+use XML::Validator::Schema::Util qw(_attr _err);
 
 sub new {
     my $pkg  = shift;
@@ -43,7 +43,7 @@ sub start_element {
         # 'unqualified' if declared since that's all we're up to
         for (qw(elementFormDefault attributeFormDefault)) {
             my $a = _attr($data, $_);
-            $self->_err("$_ in <schema> must be 'unqualified', ".
+            _err("$_ in <schema> must be 'unqualified', ".
                         "'qualified' is not supported.")
               if $a and $a ne 'unqualified';
         }
@@ -66,11 +66,60 @@ sub start_element {
         push @{$mother->{attr} ||= []}, 
              XML::Validator::Schema::Attribute->parse($data);
     }
+
+    elsif ($name eq 'simpleType') {
+        my $name = _attr($data, 'name');
+        if ($name) {
+            _err("Named simpleType must be global.")
+              unless $mother->is_root;
+
+            # this is a named type, parse it into an SimpleTypeNode 
+            # and tell Mom about it
+            my $node = XML::Validator::Schema::SimpleTypeNode->parse($data);
+            $mother->add_daughter($node);
+            push @$node_stack, $node;
+
+        } else {
+            _err("Anonymous global simpleType not allowed.")
+              if $mother->is_root;
+
+            _err("Found <simpleType> illegally combined with <complexType>.")
+              if $mother->{is_complex};
+
+            # this is a named type, parse it into an SimpleTypeNode 
+            # and tell Mom about it
+            my $node = XML::Validator::Schema::SimpleTypeNode->parse($data);
+            $mother->add_daughter($node);
+            push @$node_stack, $node;
+
+        }
+    }
     
+    elsif ($name eq 'restriction') {
+        _err("Found <restriction> outside a <simpleType> definition.")
+          unless $mother->isa('XML::Validator::Schema::SimpleTypeNode');
+        $mother->parse_restriction($data);
+    }
+
+    elsif ($name eq 'whiteSpace'   or 
+           $name eq 'pattern'      or 
+           $name eq 'enumeration'  or 
+           $name eq 'length'       or 
+           $name eq 'minLength'    or 
+           $name eq 'maxLength'    or 
+           $name eq 'minInclusive' or 
+           $name eq 'minExclusive' or 
+           $name eq 'maxInclusive' or 
+           $name eq 'maxExclusive') {
+        _err("Found <$name> outside a <simpleType> definition.")
+          unless $mother->isa('XML::Validator::Schema::SimpleTypeNode');
+        $mother->parse_facet($data);
+    }
+
     elsif ($name eq 'complexType') {
         my $name = _attr($data, 'name');
         if ($name) {
-            $self->_err("Named complexType must be global.")
+            _err("Named complexType must be global.")
               unless $mother->is_root;
 
             # this is a named type, parse it into an ComplexTypeNode 
@@ -81,7 +130,7 @@ sub start_element {
 
             
         } else {
-            $self->_err("Anonymous global complexType not allowed.")
+            _err("Anonymous global complexType not allowed.")
               if $mother->is_root;
 
             # anonymous complexTypes are just noted and passed on
@@ -90,19 +139,16 @@ sub start_element {
             
     }
 
-    elsif ($name eq 'sequence') {
-        $self->_err("Found a <sequence> embedded in a <choice> or an <all> which is not yet supported.") if $mother->{is_choice} or $mother->{is_all};
-        $mother->{is_sequence} = 1;
-    }
+    elsif ($name eq 'sequence' or $name eq 'choice' or $name eq 'all') {
+        # create a new node for the model
+        my $node = XML::Validator::Schema::ModelNode->parse($data);
+        
+        # add to current node's daughter list and become the current node
+        $mother->add_daughter($node);
+        push @$node_stack, $node;        
 
-    elsif ($name eq 'choice') {
-        $self->_err("Found a <choice> embedded in a <sequence> or an <all> which is not yet supported.") if $mother->{is_sequence} or $mother->{is_all};
-        $mother->{is_choice} = 1;
-    }
-
-    elsif ($name eq 'all') {
-        $self->_err("Found an <all> embedded in a <choice> or a <sequence> which is not yet supported.") if $mother->{is_sequence} or $mother->{is_all};
-        $mother->{is_all} = 1;
+        # all needs special support due to the restrictions on its use
+        $mother->{is_all} = 1 if $name eq 'all';
     }
 
     elsif ($name eq 'annotation' or $name eq 'documentation') {
@@ -111,7 +157,7 @@ sub start_element {
 
     else {
         # getting here is bad news
-        $self->_err("Unrecognized element '<$name>' found.");
+        _err("Unrecognized element '<$name>' found.");
     }
 }
 
@@ -119,9 +165,10 @@ sub end_element {
     my ($self, $data) = @_;
     my $node_stack = $self->{node_stack};
     my $node = $node_stack->[-1];
+    my $name = $data->{LocalName};
 
     # all done?
-    if ($data->{LocalName} eq 'schema') {
+    if ($name eq 'schema') {
         croak("Module done broke, man.  That element stack ain't empty!")
           unless @$node_stack == 1;
 
@@ -132,27 +179,42 @@ sub end_element {
     }
 
     # end of an element?
-    if ($data->{LocalName} eq 'element') {
+    if ($name eq 'element') {
+        ($node->daughters)[0]->compile 
+          if $node->daughters and 
+            ($node->daughters)[0]->isa('XML::Validator::Schema::ModelNode');
         pop @$node_stack;
         return;
     }         
 
+    # end of a model?
+    if ($name eq 'sequence' or $name eq 'choice' or $name eq 'all') {
+        pop @$node_stack;
+        return;
+    }
+
     # end of a named complexType?
-    if ($data->{LocalName} eq 'complexType' and 
-        $node_stack->[-1]->isa('XML::Validator::Schema::ComplexTypeNode')) {
+    if ($name eq 'complexType' and 
+        $node->isa('XML::Validator::Schema::ComplexTypeNode')) {
+        ($node->daughters)[0]->compile 
+          if $node->daughters and 
+            ($node->daughters)[0]->isa('XML::Validator::Schema::ModelNode');
         pop @{$self->{node_stack}};
         return;
-    }         
+    }
 
+    # end of named simpleType?
+    if ($name eq 'simpleType' and 
+        $node->isa('XML::Validator::Schema::SimpleTypeNode')) {
+        my $type = $node->compile();
+        $node->mother->{type} = $type unless $node->{name};
+        $node->mother->remove_daughter($node);
+        pop @{$self->{node_stack}};
+        return;
+    }
+        
     # it's ok to fall off the end here, not all elements recognized in
     # start_element need finalizing.
 }
 
-# throw a Validator exception.  Do I care that this sub appears in the
-# stack trace?  Unsure.
-sub _err {
-    my $self = shift;
-    XML::SAX::Exception::Validator->throw(Message => shift);
-}
-
 1;
